import { Ref } from "vue";
export { linear, easeInQuad, easeOutQuad, easeInOutQuad, easeInCubic, easeOutCubic, easeInOutCubic, easeInQuart, easeOutQuart, easeInOutQuart, easeInQuint, easeOutQuint, easeInOutQuint, tweenCallback, } from "femtotween";
export interface TweenOptions {
    /**
     * Time in ms for the tween - default: `400`
     */
    time?: number;
    /**
     * Callback function called when tweening is done.
     */
    done?: () => void;
    /**
     * Function used for easing - default: `easeInOutQuart`. See {@link https://github.com/pearofducks/femtoTween/blob/master/ease.js femtoTween easing functions}.
     */
    easeFunc?: (value: number) => number;
    /**
     * Optional transform/truncation of tweened value to a specified precision. See {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/toFixed Number.toFixed()}
     */
    precision?: number;
}
export declare type Stoppable = {
    stop: () => void;
};
export declare type TweenValue<T extends number | number[]> = Ref<T> & Stoppable;
/**
 * Composable to use {@link https://github.com/pearofducks/femtoTween femtoTween's} tween function in a (Vue 3 Composition API) reactive way
 *
 * @param source The Ref<number | number[]> off of which to trigger tweening
 * @param options Tweening options
 * @param callback Callback function that will be called on each tween-frame
 * @returns Ref that updates on each tween-frame; also exposes a 'stop' function to stop tweening
 */
export declare const useTween: <T extends number | number[]>(source: Ref<T>, options: TweenOptions, callback?: ((newValue: number) => void) | undefined) => TweenValue<T>;
