"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTween = exports.easeInOutQuint = exports.easeOutQuint = exports.easeInQuint = exports.easeInOutQuart = exports.easeOutQuart = exports.easeInQuart = exports.easeInOutCubic = exports.easeOutCubic = exports.easeInCubic = exports.easeInOutQuad = exports.easeOutQuad = exports.easeInQuad = exports.linear = void 0;
const vue_1 = require("vue");
const femtotween_1 = require("femtotween");
var femtotween_2 = require("femtotween");
Object.defineProperty(exports, "linear", { enumerable: true, get: function () { return femtotween_2.linear; } });
Object.defineProperty(exports, "easeInQuad", { enumerable: true, get: function () { return femtotween_2.easeInQuad; } });
Object.defineProperty(exports, "easeOutQuad", { enumerable: true, get: function () { return femtotween_2.easeOutQuad; } });
Object.defineProperty(exports, "easeInOutQuad", { enumerable: true, get: function () { return femtotween_2.easeInOutQuad; } });
Object.defineProperty(exports, "easeInCubic", { enumerable: true, get: function () { return femtotween_2.easeInCubic; } });
Object.defineProperty(exports, "easeOutCubic", { enumerable: true, get: function () { return femtotween_2.easeOutCubic; } });
Object.defineProperty(exports, "easeInOutCubic", { enumerable: true, get: function () { return femtotween_2.easeInOutCubic; } });
Object.defineProperty(exports, "easeInQuart", { enumerable: true, get: function () { return femtotween_2.easeInQuart; } });
Object.defineProperty(exports, "easeOutQuart", { enumerable: true, get: function () { return femtotween_2.easeOutQuart; } });
Object.defineProperty(exports, "easeInOutQuart", { enumerable: true, get: function () { return femtotween_2.easeInOutQuart; } });
Object.defineProperty(exports, "easeInQuint", { enumerable: true, get: function () { return femtotween_2.easeInQuint; } });
Object.defineProperty(exports, "easeOutQuint", { enumerable: true, get: function () { return femtotween_2.easeOutQuint; } });
Object.defineProperty(exports, "easeInOutQuint", { enumerable: true, get: function () { return femtotween_2.easeInOutQuint; } });
const extendRefWithStop = (source, stop) => {
    return Object.defineProperty(source, "stop", {
        value: stop,
        enumerable: true,
        writable: true,
    });
};
const setupTween = (from, to, options, callback) => {
    const tweeningValue = (0, vue_1.ref)(from);
    const reactivityCallback = (newValue) => {
        const truncatedValue = null == options.precision
            ? newValue
            : Number.parseFloat(newValue.toFixed(options.precision));
        if (truncatedValue !== tweeningValue.value) {
            tweeningValue.value = truncatedValue;
        }
        callback === null || callback === void 0 ? void 0 : callback(newValue);
    };
    const stop = (0, femtotween_1.tween)(from, to, reactivityCallback, options);
    return extendRefWithStop(tweeningValue, stop);
};
const useTweenEx = (from, to, options, callback) => {
    const tweenValue = (0, vue_1.ref)(from.value);
    let innerTweenValue = (0, vue_1.ref)(from.value);
    let innerTweenValueWatchHandler;
    (0, vue_1.watch)([from, to], (value, oldValue, onCleanup) => {
        onCleanup(() => {
            var _a;
            innerTweenValueWatchHandler === null || innerTweenValueWatchHandler === void 0 ? void 0 : innerTweenValueWatchHandler();
            (_a = tweenValue.stop) === null || _a === void 0 ? void 0 : _a.call(tweenValue);
        });
        innerTweenValue = setupTween(from.value, to.value, options, callback);
        tweenValue.stop = innerTweenValue.stop;
        innerTweenValueWatchHandler = (0, vue_1.watch)(innerTweenValue, () => {
            tweenValue.value = innerTweenValue.value;
        }, { immediate: true });
    }, { immediate: true });
    return tweenValue;
};
const _useTween = (source, options, callback) => {
    const from = (0, vue_1.ref)(source.value);
    const to = (0, vue_1.ref)(source.value);
    let tweenedValue = null;
    (0, vue_1.watch)(source, () => {
        if (tweenedValue) {
            from.value = tweenedValue.value;
            to.value = source.value;
        }
    }, { immediate: true });
    tweenedValue = useTweenEx(from, to, options, callback);
    return tweenedValue;
};
/**
 * Composable to use {@link https://github.com/pearofducks/femtoTween femtoTween's} tween function in a (Vue 3 Composition API) reactive way
 *
 * @param source The Ref<number | number[]> off of which to trigger tweening
 * @param options Tweening options
 * @param callback Callback function that will be called on each tween-frame
 * @returns Ref that updates on each tween-frame; also exposes a 'stop' function to stop tweening
 */
const useTween = (source, options, callback) => {
    if ("number" === typeof source.value) {
        return _useTween(source, options, callback);
    }
    const sourceValues = source.value.map((value) => (0, vue_1.ref)(value));
    const tweenedValues = (0, vue_1.reactive)(sourceValues.map((value) => {
        return _useTween(value, options, callback);
    }));
    const unWrappedTweenedValues = (0, vue_1.computed)(() => {
        return tweenedValues.map((value) => value.value);
    });
    (0, vue_1.watch)(source, (newSources) => {
        newSources.forEach((value, index) => {
            sourceValues[index].value = value;
        });
    }, { deep: true });
    return extendRefWithStop(unWrappedTweenedValues, () => {
        tweenedValues.forEach((value) => {
            value.stop();
        });
    });
};
exports.useTween = useTween;
